'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var emotionUtils = require('emotion-utils');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*

high performance StyleSheet for css-in-js systems

- uses multiple style tags behind the scenes for millions of rules
- uses `insertRule` for appending in production for *much* faster performance
- 'polyfills' on server side

// usage

import StyleSheet from 'glamor/lib/sheet'
let styleSheet = new StyleSheet()

styleSheet.inject()
- 'injects' the stylesheet into the page (or into memory if on server)

styleSheet.insert('#box { border: 1px solid red; }')
- appends a css rule into the stylesheet

styleSheet.flush()
- empties the stylesheet of all its contents

*/

function sheetForTag(tag) {
  if (tag.sheet) {
    return tag.sheet;
  }

  // this weirdness brought to you by firefox
  for (var i = 0; i < document.styleSheets.length; i++) {
    if (document.styleSheets[i].ownerNode === tag) {
      return document.styleSheets[i];
    }
  }
}

var isBrowser = typeof window !== 'undefined';

function makeStyleTag() {
  var tag = document.createElement('style');
  tag.type = 'text/css';
  tag.setAttribute('data-emotion', '');
  tag.appendChild(document.createTextNode(''));
  document.head.appendChild(tag);
  return tag;
}

var StyleSheet = function () {
  function StyleSheet() {
    _classCallCheck(this, StyleSheet);

    this.isSpeedy = process.env.NODE_ENV === 'production'; // the big drawback here is that the css won't be editable in devtools
    this.tags = [];
    this.ctr = 0;
  }

  StyleSheet.prototype.inject = function inject() {
    if (this.injected) {
      throw new Error('already injected!');
    }
    if (isBrowser) {
      this.tags[0] = makeStyleTag();
    } else {
      // server side 'polyfill'. just enough behavior to be useful.
      this.sheet = {
        cssRules: []
      };
    }
    this.injected = true;
  };

  StyleSheet.prototype.speedy = function speedy(bool) {
    if (this.ctr !== 0) {
      // cannot change speedy mode after inserting any rule to sheet. Either call speedy(${bool}) earlier in your app, or call flush() before speedy(${bool})
      throw new Error('cannot change speedy now');
    }
    this.isSpeedy = !!bool;
  };

  StyleSheet.prototype.insert = function insert(rule) {
    if (isBrowser) {
      var tag = this.tags[this.tags.length - 1];
      var sheet = sheetForTag(tag);
      // this is the ultrafast version, works across browsers
      if (this.isSpeedy && sheet.insertRule) {
        // this weirdness for perf, and chrome's weird bug
        // https://stackoverflow.com/questions/20007992/chrome-suddenly-stopped-accepting-insertrule
        try {
          sheet.insertRule(rule, sheet.cssRules.length);
        } catch (e) {
          if (process.env.NODE_ENV !== 'production') {
            // might need beter dx for this
            console.warn('illegal rule', rule); // eslint-disable-line no-console
          }
        }
      } else {
        // more browser weirdness. I don't even know
        // else if(this.tags.length > 0 && this.tags::last().styleSheet) {
        //   this.tags::last().styleSheet.cssText+= rule
        // }
        tag.appendChild(document.createTextNode(rule));
      }
    } else {
      // enough 'spec compliance' to be able to extract the rules later
      // in other words, just the cssText field
      this.sheet.cssRules.push({ cssText: rule });
    }

    this.ctr++;
    if (isBrowser && this.ctr % 65000 === 0) {
      this.tags.push(makeStyleTag());
    }
    return this.ctr - 1;
  };

  StyleSheet.prototype.flush = function flush() {
    if (isBrowser) {
      this.tags.forEach(function (tag) {
        return tag.parentNode.removeChild(tag);
      });
      this.tags = [];
      this.ctr = 0;
      // todo - look for remnants in document.styleSheets
    } else {
      // simpler on server
      this.sheet.cssRules = [];
    }
    this.injected = false;
  };

  return StyleSheet;
}();

var sheet = new StyleSheet();
// 🚀
sheet.inject();
var stylisOptions = { keyframe: false };

var stylis = new emotionUtils.Stylis(stylisOptions);
var keyframeStylis = new emotionUtils.Stylis(stylisOptions);

exports.registered = {};

exports.inserted = {};

function flush() {
  sheet.flush();
  exports.inserted = {};
  exports.registered = {};
  sheet.inject();
}

function insertionPlugin(context, content, selector, parent) {
  switch (context) {
    case 2:
      {
        if (parent[0] === selector[0]) {
          break;
        }
      }
    // after an at rule block
    case 3:
      // eslint-disable-line no-fallthrough
      sheet.insert(selector.join(',') + '{' + content + '}');
  }
}

function keyframeInsertionPlugin(context, content, selector) {
  if (context === 3) {
    sheet.insert(selector[0].replace('keyframes', '-webkit-keyframes') + '{' + content + '}');
    sheet.insert(selector[0] + '{' + content + '}');
  }
}

stylis.use(insertionPlugin);
keyframeStylis.use(keyframeInsertionPlugin);

function flatten(inArr) {
  var arr = [];
  inArr.forEach(function (val) {
    if (Array.isArray(val)) arr = arr.concat(flatten(val));else arr = arr.concat(val);
  });

  return arr;
}

function handleInterpolation(interpolation, couldBeSelectorInterpolation) {
  if (typeof interpolation === 'object') {
    return createStringFromObject(interpolation);
  }
  if (interpolation === undefined || interpolation === null || interpolation === false) return '';
  if (couldBeSelectorInterpolation === false && exports.registered[interpolation] !== undefined) {
    return exports.registered[interpolation];
  }
  return interpolation;
}

var hyphenateRegex = /[A-Z]|^ms/g;

var processStyleName = emotionUtils.memoize(function (styleName) {
  return styleName.replace(hyphenateRegex, '-$&').toLowerCase();
});

var processStyleValue = function processStyleValue(key, value) {
  if (value === undefined || value === null || typeof value === 'boolean') return '';

  if (emotionUtils.unitless[key] !== 1 && !isNaN(value) && value !== 0) {
    return value + 'px';
  }
  return value;
};

function createStringFromObject(obj) {
  var string = '';

  if (Array.isArray(obj)) {
    flatten(obj).forEach(function (interpolation) {
      string += handleInterpolation(interpolation, false);
    });
  } else {
    Object.keys(obj).forEach(function (key) {
      if (typeof obj[key] !== 'object') {
        if (exports.registered[obj[key]] !== undefined) {
          string += key + '{' + exports.registered[obj[key]] + '}';
        } else {
          string += processStyleName(key) + ':' + processStyleValue(key, obj[key]) + ';';
        }
      } else {
        string += key + '{' + createStringFromObject(obj[key]) + '}';
      }
    });
  }
  return string;
}

function isLastCharDot(string) {
  return string.charCodeAt(string.length - 1) === 46; // .
}

function createStyles(strings) {
  var stringMode = true;
  var styles = '';
  if (strings !== undefined && strings.raw === undefined || strings === undefined) {
    stringMode = false;
    styles = handleInterpolation(strings, false);
  } else {
    styles = strings[0];
  }

  for (var _len = arguments.length, interpolations = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    interpolations[_key - 1] = arguments[_key];
  }

  interpolations.forEach(function (interpolation, i) {
    styles += handleInterpolation(interpolation, isLastCharDot(styles));
    if (stringMode === true && strings[i + 1] !== undefined) {
      styles += strings[i + 1];
    }
  });
  return styles;
}

function css() {
  var styles = createStyles.apply(undefined, arguments);
  var hash = emotionUtils.hashString(styles);
  var cls = 'css-' + hash;
  if (exports.registered[cls] === undefined) {
    exports.registered[cls] = styles;
  }
  if (exports.inserted[hash] === undefined) {
    stylis('.' + cls, styles);
    exports.inserted[hash] = true;
  }
  return cls;
}

function injectGlobal() {
  var styles = createStyles.apply(undefined, arguments);
  var hash = emotionUtils.hashString(styles);
  if (exports.inserted[hash] === undefined) {
    stylis('', styles);
    exports.inserted[hash] = true;
  }
}

function hydrate(ids) {
  ids.forEach(function (id) {
    exports.inserted[id] = true;
  });
}

function keyframes() {
  var styles = createStyles.apply(undefined, arguments);
  var hash = emotionUtils.hashString(styles);
  var name = 'animation-' + hash;
  if (exports.inserted[hash] === undefined) {
    keyframeStylis('', '@keyframes ' + name + '{' + styles + '}');
    exports.inserted[hash] = true;
  }
  return name;
}

function fontFace() {
  var styles = createStyles.apply(undefined, arguments);
  var hash = emotionUtils.hashString(styles);
  if (exports.inserted[hash] === undefined) {
    sheet.insert('@font-face{' + styles + '}');
    exports.inserted[hash] = true;
  }
}

function getRegisteredStyles(registeredStyles, classNames) {
  var rawClassName = '';

  classNames.split(' ').forEach(function (className) {
    if (exports.registered[className] !== undefined) {
      registeredStyles.push(className);
    } else {
      rawClassName += className + ' ';
    }
  });
  return rawClassName;
}

function merge(className) {
  var registeredStyles = [];

  var rawClassName = getRegisteredStyles(registeredStyles, className);

  if (registeredStyles.length < 2) {
    return className;
  }
  return rawClassName + css.apply(undefined, registeredStyles);
}

exports.sheet = sheet;
exports.flush = flush;
exports.css = css;
exports.injectGlobal = injectGlobal;
exports.hydrate = hydrate;
exports.keyframes = keyframes;
exports.fontFace = fontFace;
exports.getRegisteredStyles = getRegisteredStyles;
exports.merge = merge;
//# sourceMappingURL=index.cjs.js.map
